/*For a Nixie clock prototype - Alnwlsn 2017
 *Clock uses two 74hc595 shift registers in series to control four nixie tubes via for k155id1 driver ics.  
 *Most signifigant information sent out first, so thousands tube gets first four bits, ones tube gets last four bits, etc.
 *DS3231 for timekeeping.
 *
 *Clock normally displays the time but from 2 - 5 am it cycles all nixie digits to reduce cathode poisioning.
 *
 *Connections (arduino - item)
 * 2 - pull low to decrement time
 * 3 - pull low to increment time
 * 4 - pull low to send time output over serial
 * 5 - pull low to cycle all digits
 * 7  -  to ds (data pin) of first shift reg
 * 8  -  to shcp (clock) of first shift reg
 * 12 -  to stcp (latch) of first shift reg 
 * A4 - SDA on ds3231
 * A5 - SCL on ds3231
 */
#include <Wire.h>
#include "RTClib.h"

#define DS    7 //data pin
#define SHCP  8 //clock pin
#define STCP  12 //latch pin

RTC_DS3231 rtc;

unsigned int crtime = 0; //the current time
unsigned int lstime = 0; //last time
unsigned long unixref = 0;
int jiffyoffset = 0;
byte zcth = 0;
byte mode = 0;

DateTime now;

void setup () {
  pinMode(DS, OUTPUT);
  pinMode(STCP, OUTPUT);
  pinMode(SHCP, OUTPUT);
  pinMode(2, INPUT_PULLUP); //time decrease button
  pinMode(3, INPUT_PULLUP); //time increase button
  pinMode(4, INPUT_PULLUP); //serial time output enable
  pinMode(5, INPUT_PULLUP); //cycle all digits
  pinMode(6, INPUT_PULLUP); //wintercamp jiffy mode
  pinMode(13, OUTPUT);      //general purpose indicator
  Serial.begin(9600);
  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }

rtc.writeSqwPinMode(0x00); //set ds3231 to output 1hz square wave
//rtc.adjust(DateTime(F(__DATE__), F(__TIME__))+TimeSpan(0,0,0,8)); //set time

  DateTime now = rtc.now();
  crtime = now.hour()*100 + now.minute();
  nixie(crtime);

}

void loop () {
  now = rtc.now();
  if(digitalRead(6)==0){
    jiffyClock();
  } else {
    if(mode != 0){ mode = 0; }
    ZhrClock();
  }
  byte czcth = 0;
  if(now.hour()>=2 && now.hour()<=4){ czcth = 1; }
  if(digitalRead(5)==0){ czcth = 2; }
  if(czcth != 0){  //cycle through digits for couple hours when nobody's looking
    czcth = 0;
    zcth++;
    if(zcth >= 10){ zcth = 0; }
    int k = 1000*zcth+100*zcth+10*zcth+zcth;
    nixie(k);
    delay(4 00);
  } else {
    nixie(now.hour()*100 + now.minute());
  }
  delay(100);
}

void jiffyClock(){
  if(unixref == 0){ unixref = now.unixtime();}
  unsigned long jiffytimesec = now.unixtime() - unixref;
  float jiffytime = jiffytimesec/93.9963716499; //conversion from seconds to jiffies
  int z = (int)jiffytime + jiffyoffset;
  if(digitalRead(3)==0){      //decrease
    unsigned int cycle = 0;
    while(digitalRead(3)==0){
      z--;
      jiffyoffset = z;
      unixref = now.unixtime();
      nixie(z);
      if(cycle >= 0 && cycle < 1    ){delay(1000);}
      if(cycle >= 1 && cycle < 11   ){delay(500);}
      if(cycle >= 11 && cycle < 61  ){delay(100);}
      if(cycle >= 61                ){delay(10);}
      cycle++;
    }
  }
  if(digitalRead(2)==0){
    unsigned int cycle = 0;
    while(digitalRead(2)==0){ //increase
      z++;
      jiffyoffset = z;
      unixref = now.unixtime();
      nixie(z);
      if(cycle >= 0 && cycle < 1    ){delay(1000);}
      if(cycle >= 1 && cycle < 11   ){delay(500);}
      if(cycle >= 11 && cycle < 61  ){delay(100);}
      if(cycle >= 61                ){delay(10);}
      cycle++;
    }
  }
  nixie(z);
}

void ZhrClock(){
    if(digitalRead(3)==0){
    unsigned int cycle = 0;
    while(digitalRead(3)==0){
      now = rtc.now();
      
      if(cycle >= 0 && cycle < 1     ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)+TimeSpan(0,0,1,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(1000); }
      if(cycle >= 1 && cycle < 16    ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)+TimeSpan(0,0,1,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(200);  }
      if(cycle >= 16 && cycle < 61   ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)+TimeSpan(0,0,1,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(100);  }
      if(cycle >= 61 && cycle < 361  ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)+TimeSpan(0,0,1,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(10);   }
      if(cycle >= 361 && cycle < 371 ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)+TimeSpan(0,1,0,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(500);  }
      if(cycle > 371                 ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)+TimeSpan(0,1,0,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(100);  }
      cycle++;
    }
    delay(100);
  }
  if(digitalRead(2)==0){
    unsigned int cycle = 0;
    while(digitalRead(2)==0){
      now = rtc.now();
      
      if(cycle >= 0 && cycle < 1     ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)-TimeSpan(0,0,1,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(1000); }
      if(cycle >= 1 && cycle < 16    ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)-TimeSpan(0,0,1,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(200);  }
      if(cycle >= 16 && cycle < 61   ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)-TimeSpan(0,0,1,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(100);  }
      if(cycle >= 61 && cycle < 361  ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)-TimeSpan(0,0,1,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(10);   }
      if(cycle >= 361 && cycle < 371 ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)-TimeSpan(0,1,0,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(500);  }
      if(cycle > 371                 ){ rtc.adjust(DateTime(now.year(), now.month(), now.day(), now.hour(), now.minute(), 0)-TimeSpan(0,1,0,0));  now = rtc.now(); nixie(now.hour()*100 + now.minute()); srtime(); delay(100);  }
      cycle++;
    }
    delay(100);
  }
  
  srtime();
}


void nixie(int number){
    if(number < 0){digitalWrite(13, HIGH);}else{digitalWrite(13, LOW);}
    number = abs(number);
    byte thou = number / 1000;
    number = number-thou*1000;
    byte hund = number / 100;
    number = number-hund*100;
    byte tens = number / 10;
    number = number-tens*10;
    byte ones = number;

    ones = ones + 16*tens;  //combined ones and tens
    hund = hund + 16*thou;  //combined hundreds and thousands

    digitalWrite(STCP, LOW); //latch low
    shiftOut(DS, SHCP, MSBFIRST, hund);
    shiftOut(DS, SHCP, MSBFIRST, ones);
    digitalWrite(STCP, HIGH); //latch high
}

void srtime() {
  if(digitalRead(4)==0){
    Serial.print(now.year(), DEC);
    Serial.print('/');
    Serial.print(now.month(), DEC);
    Serial.print('/');
    Serial.print(now.day(), DEC);
    Serial.print(' ');
    Serial.print(now.dayOfTheWeek());
    Serial.print(' ');
    Serial.print(now.hour(), DEC);
    Serial.print(':');
    Serial.print(now.minute(), DEC);
    Serial.print(':');
    Serial.print(now.second(), DEC);
    Serial.println();
  }
}

