 /*For a Nixie clock prototype - Alnwlsn 2018 APR - 6 tube, IR version
 *Clock uses three 74hc595 shift registers in series to control six nixie tubes via for k155id1 driver ics.  
 *Most signifigant information sent out first.
 *ds3231 for timekeeping.
 *
 *Clock normally displays the time but from 2 - 6:59:59 am it cycles all nixie digits to reduce cathode poisioning.
 *
 *Connections (arduino - item)
 * A4 - SDA on odatapin3231
 * A5 - SCL on odatapin3231
 * D2 - IR sensor (use the NEC remotes that come with RTLSDRs) Uses interrupts.
 */
 
#include <Wire.h>
#include "RTClib.h"
#include <EEPROM.h>
#include <NewTone.h>
#include <MemoryFree.h>

#include <IRLibDecodeBase.h>
#include <IRLib_P04_RC6.h>
#include <IRLibCombo.h> 
#include <IRLibRecv.h> 

#define odatapin     10
#define oclockpin    11
#define olatchpin    12
#define ilatchpin     5
#define iclockpin     6
#define idatapin      7
#define buzzerpin     9
#define syncpin       A0
#define WWVBpin       A1

RTC_DS3231 rtc;
IRdecode myDecoder;
IRrecv myReceiver(2); //ir reciver on pin 2

byte localoffsethour = 127;    //stores offset minute and second for conversion from UTC to Local
byte localoffsetminute = 127;  //

byte conhour = 0; //if local hour is greater than this
byte coffhour = 0;//but less than this, cathode poisoning routine will run

byte scycle = 0;  //variable that keeps track of things that happen every second vs 20x second.
byte ccycle = 0;  //digit for cathode poisoning routine
byte cdps = 0;    //cathode poising routine - turns decimal points on and off

byte channel = 1; //the current "channel" that is normally displayed on the nixie tubes (on startup)

byte neon = 0;    //bit0 control left neon, bit1 control right neon. Updated by nixie(). Overrides decimal.

byte pause = 0;   //if nonzero, pause cathode digit cycling.

byte daydec1 = 0;  //sets number of decimal places on right for days (for reference 1)
byte jifdec1 = 0;  //sets number of decimal places for jiffies 
byte daydec2 = 0;  //sets number of decimal places on right for days (for reference 2)
byte jifdec2 = 0;  //sets number of decimal places for jiffies

int64_t stopwatch_start = 0;
unsigned long stopwatch_t = 0;
boolean stopwatch_run = 0;

unsigned long lastmillis = 0;
char currentsecond = -1;

byte beeping = 0;
bool isBeeping=0;

bool ircyclelast = 0;

DateTime utc;
DateTime local;
DateTime reference1;
DateTime reference2;

void setup (){
  pinMode(odatapin, OUTPUT);
  pinMode(oclockpin, OUTPUT);
  pinMode(olatchpin, OUTPUT);
  pinMode(ilatchpin, OUTPUT);
  pinMode(iclockpin, OUTPUT);
  pinMode(idatapin, INPUT);
  pinMode(buzzerpin, OUTPUT);
  pinMode(syncpin, INPUT_PULLUP);
  pinMode(WWVBpin, INPUT_PULLUP);
  
  Serial.begin(115200);
  myReceiver.enableIRIn(); // Start the receiver

  rtc.begin();
  rtc.writeSqwPinMode(0x00); //set odatapin3231 to output 1hz square wave
  //rtc.adjust(DateTime(F(__DATE__), F(__TIME__))+TimeSpan(0,0,0,8)); //set time
  utc = rtc.now();

  digitalWrite(iclockpin, LOW);
  digitalWrite(ilatchpin, HIGH);
  readconfig();

  Serial.println(F("Settings:"));
  Serial.print(F("localoffsethour   : ")); Serial.println(localoffsethour-127);
  Serial.print(F("localoffsetminute : ")); Serial.println(localoffsetminute-127);
  Serial.print(F("conhour           : ")); Serial.println(conhour);
  Serial.print(F("coffhour          : ")); Serial.println(coffhour);
  Serial.print(F("reference1        : ")); Serial.print(reference1.year());Serial.print(F("/"));Serial.print(reference1.month());Serial.print(F("/"));Serial.print(reference1.day());Serial.print(F(" "));Serial.print(reference1.hour());Serial.print(F(":"));Serial.print(reference1.minute());Serial.print(F(":"));Serial.println(reference1.second());
  
  stopwatch_start = utc.unixtime()*10 + cmillis()/100;
}

void loop(){
  utc = rtc.now();
  if(currentsecond != utc.second()){ lastmillis = millis(); currentsecond = utc.second(); }
  local = DateTime(utc + TimeSpan(0,localoffsethour-127,localoffsetminute-127,0));
  if(stopwatch_run){stopwatch_t = utc.unixtime()*10 + cmillis()/100 - stopwatch_start;}

  if(beeping>0){isBeeping=1;}
  switch(beeping){
    case 1:
      if(millis()%150<75){NewTone(buzzerpin, 1500);bitWrite(neon,7,1);}else{noNewTone(buzzerpin);bitWrite(neon,7,0);}
      break;
    case 2:
      NewTone(buzzerpin, 1500);
      break;
    default:
      if(isBeeping==1){noNewTone(buzzerpin);neon=0;}
      isBeeping=0;
      break;
  }
  
  if(local.hour()>=conhour && local.hour()<=coffhour && pause==0){                   //cycles digits at night or whenever
    cycledigits();
  }else{
    switch(channel){
      case 0: //UTC time
        nixie(utc.hour()*10000L+utc.minute()*100L+utc.second(), 0);
        break;
      case 1: //Local Time
        nixie(local.hour()*10000L+local.minute()*100L+local.second(), 0);
        break;
      case 2: //Winter Camp Time
        local = DateTime(utc + TimeSpan(0,-3,-43,0));  //can use local since it will be overwritten on next loop.
        nixie(local.hour()*10000L+local.minute()*100L+local.second(), 0);
        break;
      case 3: //Jiffies until next Winter Camp
        {DateTime tempdate = DateTime(utc.year(),12,27,15,43,0); //Start date 12,27,15,43,0 wintercamp start
        DateTime tempdate2 = DateTime(utc.year(),12,31,20,0,0);  //End date   12,31,20,0,0  wintercamp end
        if(utc.unixtime() >= tempdate2.unixtime()){tempdate = DateTime(utc.year()+1,12,27,15,43,0);} //alt Start date
        long diff = (utc.unixtime()*10 + cmillis()/100 - tempdate.unixtime()*10);
        float temp1 = floor(abs(diff / (9399.63716499)));
        byte tempdp = 4;
        if(temp1 >= 100){ tempdp = 3; }
        if(temp1 >= 1000){ tempdp = 2; }
        if(temp1 >= 10000){ tempdp = 1; }
        float temp = floor(diff / (9399.63716499*pow(10, -tempdp)));
        byte decimalmask;
        if(tempdp>0){bitSet(decimalmask, tempdp-1);}
        nixie(temp, decimalmask);}
        break;
      case 4: //TEST - Cycle digits  
        cycledigits();
        break;
      case 5: //Stopwatch display
        nixie(stopwatch_t, 1);
        break; 
      default: //default is local time
        channel = 1; 
        break;
    }  
  }
  
  byte inkey = irCheck();
  switch(inkey){
      case 73: //pictures
        channel = 4; //cycle digits test
        break;
      case 37: //livetv
        channel = 0; //UTC time
        break;
      case 12: //power
        {if(pause==0){pause=30;}else{pause=0;}}
        break;
      case 13:
        {Serial.println(F("Maintenance"));
        byte record = getIRlong(0);
        Serial.print(F("Selected record ")); Serial.println(record);
        bitSet(neon, 7);
        switch(record){
          case 1: //set time
            {byte temph=utc.hour();
             byte tempm=utc.minute();
             byte temps=utc.second();
            long temp = getIRlong(temph*10000L+tempm*100L+temps);
            utc = rtc.now();
            temph=temp/10000L;
            tempm=(temp-temph*10000L)/100L;
            temps=temp-temph*10000L-tempm*100L;
            rtc.adjust(DateTime(utc.year(),utc.month(),utc.day(),temph,tempm,temps));
            Serial.print(F("Clock time set: "));
            Serial.print(temph);Serial.print(':');Serial.print(tempm);Serial.print(':');Serial.println(temps);
            }
            break;
          case 2: //set date
            {byte tempy=utc.year()-2000;
             byte tempm=utc.month();
             byte tempd=utc.day();
            long temp = getIRlong(tempy*10000L+tempm*100L+tempd);
            utc = rtc.now();
            tempy=temp/10000L;
            tempm=(temp-tempy*10000L)/100L;
            tempd=temp-tempy*10000L-tempm*100L;
            rtc.adjust(DateTime(tempy+2000,tempm,tempd,utc.hour(),utc.minute(),utc.second()));
            Serial.print(F("Clock date set: "));
            Serial.print(tempy+2000);Serial.print('/');Serial.print(tempm);Serial.print('/');Serial.println(tempd);
            }
            break;
          case 3: //set offset
            {int temph=localoffsethour-127;
             int tempm=localoffsetminute-127;
            long temp = getIRlong(temph*100L+tempm);
            temph=temp/100L;
            tempm=(temp-temph*100L);
            Serial.print(F("UTC to local offset: "));
            Serial.print(temph);Serial.print(':');Serial.println(abs(tempm));
            localoffsethour = 127+temph;
            localoffsetminute = 127+tempm;
            }
            break;
          case 9:  {conhour  = getIRlong(conhour);  Serial.print(F("Cathode poisoning on set: "));  Serial.println(conhour); } break;
          case 10: {coffhour = getIRlong(coffhour); Serial.print(F("Cathode poisoning off set: ")); Serial.println(coffhour);} break;
          case 11: secondsAdjust(); break;
          case 12:
            {int temp = getIRlong(0);
            NewTone(buzzerpin, temp, 1000);delay(1000);}
            break;
         case 60:
           nixie(0, 192);
           WWVBsync();
           break;
         case 254:
           readconfig();
           break;
         case 255:
           writeconfig();
           break;
         }
        bitClear(neon, 7);
        NewTone(buzzerpin, 500, 100);
        }break;
      case 74: //videos
        channel = 2; // Winter Camp Time
        break;
      case 22: //play - reset and start stopwatch
        stopwatch_start = utc.unixtime()*10 + cmillis()/100;
        stopwatch_run = 1;
        channel = 5;
        break;
      case 70: //TV 
        channel = 1; //local time
        break;
      case 24: //pause //start/stop stopwatch
        stopwatch_run = !stopwatch_run;
        channel = 5;
        break;
      case 72: //recordedtv
        channel = 3; //Jiffy clock
        break;
    }

  
  scycle++;
  
  if(scycle==20){ scycle = 0; //runs every second
    if(pause>0){pause--;}
    //Serial.print(F("Local: "));Serial.print(local.year(), DEC);Serial.print('/');Serial.print(local.month(), DEC);Serial.print('/');Serial.print(local.day(), DEC);Serial.print(" ");Serial.print(local.hour(), DEC);Serial.print(':');Serial.print(local.minute(), DEC);Serial.print(':');Serial.println(local.second(), DEC);  
    //Serial.print(F("UTC:   "));Serial.print(utc.year(), DEC);Serial.print('/');Serial.print(utc.month(), DEC);Serial.print('/');Serial.print(utc.day(), DEC);Serial.print(" ");Serial.print(utc.hour(), DEC);Serial.print(':');Serial.print(utc.minute(), DEC);Serial.print(':');Serial.print(utc.second(), DEC);Serial.print(' ');Serial.println(utc.unixtime());
    //Serial.print(F("Ref:   "));Serial.print(reference.year(), DEC);Serial.print('/');Serial.print(reference.month(), DEC);Serial.print('/');Serial.print(reference.day(), DEC);Serial.print(" ");Serial.print(reference.hour(), DEC);Serial.print(':');Serial.print(reference.minute(), DEC);Serial.print(':');Serial.print(reference.second(), DEC);Serial.print(' ');Serial.println(reference.unixtime());
    //Serial.println(diff, DEC);
  }
  
  delay(20);
}

void nixie(long number, byte decimal){
    if(bitRead(neon, 6)){bitSet(decimal, 6);} //left neon
    if(bitRead(neon, 7)){bitSet(decimal, 7);} //right neon
    if(number < 0){bitSet(decimal, 6);}
    number = abs(number);
    byte hthou = number / 100000L;
    number = number-hthou*100000L;
    byte tthou = number / 10000L;
    number = number-tthou*10000L;
    byte thou = number / 1000L;
    number = number-thou*1000L;
    byte hund = number / 100L;
    number = number-hund*100L;
    byte tens = number / 10L;
    number = number-tens*10L;
    byte ones = number;

    ones = ones + 16*tens;  //combined ones and tens
    hund = hund + 16*thou;
    tthou = tthou + 16*hthou;

    digitalWrite(oclockpin, LOW); //latch low
    shiftOut(odatapin, olatchpin, MSBFIRST, decimal);
    shiftOut(odatapin, olatchpin, MSBFIRST, tthou);
    shiftOut(odatapin, olatchpin, MSBFIRST, hund);
    shiftOut(odatapin, olatchpin, MSBFIRST, ones);
    digitalWrite(oclockpin, HIGH); //latch high
}

long getIRlong(long existing){
  long current = 0;
  Serial.println(F("Enter number:"));
  while(1){
    nixie(current, 0);
    Serial.println(current, DEC);
    byte inkey = 255; while(inkey == 255){ inkey = irCheck(); }
    if(inkey == 34){return current;}               //assembling the number is done, return it (OK key)
    if(inkey == 35){current = existing;}           //user pressed BACK. let current equal the existing number
    if(inkey == 10){current = 0;}                  //user pressed CLEAR. Reset to zero.
    if(inkey == 30){current++;}                    //up/down buttons add/subtract      
    if(inkey == 31){current--;}
    if(inkey == 29){current = -current;}           //user pressed * (using as +/- key);
    if(inkey < 10){if(current>=0){current = 10*current + inkey;}else{current = 10*current - inkey;}}  //a number was entered. Append it, and manage minus sign;
  }
}

void secondsAdjust(){
  byte current = 0;
  byte neonMask = 0;
  byte inkey = 255;
  while(1){
    if(inkey == 255){ inkey = irCheck(); }
    utc = rtc.now();
    if(currentsecond != utc.second()){
      lastmillis = millis(); currentsecond = utc.second();
      if(utc.second()==0){NewTone(buzzerpin, 1000, 800);}else{NewTone(buzzerpin, 1000, 5);}
      if(inkey == 30){rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())+TimeSpan(0,0,0,1)); inkey = 255;}                    //channel up/down buttons add/subtract seconds      
      if(inkey == 31){rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())-TimeSpan(0,0,0,1)); inkey = 255;}
      }
    if(inkey == 33){
      inkey=255;
      rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second()));
      }
    if(cmillis()<50){neonMask = 64;}else{neonMask = 0;}
    nixie(utc.hour()*10000L+utc.minute()*100L+utc.second(), neonMask); 
    if(inkey == 34){digitalWrite(buzzerpin, 0); return;}               //User pressed enter. Done
  }
}

byte irCheck(){                                //Check for a valid code
  if(myReceiver.getResults()){ 
    myDecoder.decode();                        //Decode ir code
    unsigned long rxcode=myDecoder.value;
    unsigned long rxcodeprefix=rxcode/4096; //remote sends alternating prefix to differentiate holding vs next keypress
    rxcode-=rxcodeprefix*4096;
    bool ircycle=0;
    bool unique=0;
    if(rxcodeprefix==0x800F8){ircycle=1;}
    //Serial.print(F("IR signal: C"));Serial.print(ircycle);Serial.print(F(", 0x"));Serial.print(rxcode, HEX);
    if(ircycle!=ircyclelast){ircyclelast=ircycle; unique=1;}
    //if(unique==0){return 255;}
    //Serial.println(unique, HEX);
    byte decoded=rxcode-0x400;
    delay(20);
    myReceiver.enableIRIn();                   //Restart receiver
    delay(20);
    if(unique){
      /*Serial.print(F("IR code: 0x"));
      Serial.print(rxcode, HEX);
      Serial.print(F(" as #"));
      Serial.println(decoded);*/
      return decoded;
      }
    return 255;
  } else { return 255; }
}

byte read165(){
  byte rin = 0;
  digitalWrite(ilatchpin, LOW);
  delayMicroseconds(5);
  digitalWrite(ilatchpin, HIGH);
  for(byte i = 0; i < 8; i++){
    bitWrite(rin, i, digitalRead(idatapin));
    digitalWrite(iclockpin, HIGH);
    delayMicroseconds(5);
    digitalWrite(iclockpin, LOW);
  }
   return(rin);
}

void readconfig(){
  localoffsethour   = EEPROM.read(10);
  localoffsetminute = EEPROM.read(11);
  conhour  = EEPROM.read(12);
  coffhour = EEPROM.read(13);
  reference1 = DateTime(EEPROM.read(14)+2000,EEPROM.read(15),EEPROM.read(16),EEPROM.read(17),EEPROM.read(18),EEPROM.read(19));
  daydec1 = EEPROM.read(20);
  jifdec1 = EEPROM.read(21);
  reference2 = DateTime(EEPROM.read(22)+2000,EEPROM.read(23),EEPROM.read(24),EEPROM.read(25),EEPROM.read(26),EEPROM.read(27));
  daydec2 = EEPROM.read(28);
  jifdec2 = EEPROM.read(29);
  NewTone(buzzerpin, 1000, 100); delay(100);
  NewTone(buzzerpin, 1250, 100); delay(100);
  NewTone(buzzerpin, 1500, 100); delay(100);
}

void writeconfig(){
  EEPROM.update(10, localoffsethour);
  EEPROM.update(11, localoffsetminute);
  EEPROM.update(12, conhour);
  EEPROM.update(13, coffhour);
  EEPROM.update(14, reference1.year()-2000);EEPROM.update(15, reference1.month());EEPROM.update(16, reference1.day());EEPROM.update(17, reference1.hour());EEPROM.update(18, reference1.minute());EEPROM.update(19, reference1.second());
  EEPROM.update(20, daydec1);
  EEPROM.update(21, jifdec1);
  EEPROM.update(22, reference2.year()-2000);EEPROM.update(23, reference2.month());EEPROM.update(24, reference2.day());EEPROM.update(25, reference2.hour());EEPROM.update(26, reference2.minute());EEPROM.update(27, reference2.second());
  EEPROM.update(28, daydec2);
  EEPROM.update(29, jifdec2);
  NewTone(buzzerpin, 1500, 100); delay(100);
  NewTone(buzzerpin, 1250, 100); delay(100);
  NewTone(buzzerpin, 1000, 100); delay(100);
}

void cycledigits(){
  ccycle++;
     if(ccycle>=10){
       ccycle=0;
       if(cdps == 0){cdps = 0b11111111;}else{cdps = 0;}
     }
     nixie(ccycle*111111L, cdps);
     delay(500);
}

int cmillis(){
  long temp = millis() -  lastmillis;
  return temp;
}
