 /*For a Nixie clock prototype - Alnwlsn 2018 FEB - 6 tube, IR version
 *Clock uses three 74hc595 shift registers in series to control six nixie tubes via for k155id1 driver ics.  
 *Most signifigant information sent out first.
 *ds3231 for timekeeping.
 *
 *Clock normally displays the time but from 2 - 6:59:59 am it cycles all nixie digits to reduce cathode poisioning.
 *
 *Connections (arduino - item)
 * A4 - SDA on odatapin3231
 * A5 - SCL on odatapin3231
 * D2 - IR sensor (use the NEC remotes that come with RTLSDRs) Uses interrupts.
 */
 
#include <Wire.h>
#include "RTClib.h"
#include <EEPROM.h>
#include <NewTone.h>
#include <MemoryFree.h>

#include <IRLibDecodeBase.h>
#include <IRLib_P01_NEC.h>
#include <IRLibCombo.h> 
#include <IRLibRecv.h> 

#define odatapin     10
#define oclockpin    11
#define olatchpin    12
#define ilatchpin     5
#define iclockpin     6
#define idatapin      7
#define buzzerpin     9
#define syncpin       A0
#define WWVBpin       A1

RTC_DS3231 rtc;
IRdecode myDecoder;
IRrecv myReceiver(2); //ir reciver on pin 2

byte localoffsethour = 127;    //stores offset minute and second for conversion from UTC to Local
byte localoffsetminute = 127;  //

byte conhour = 0; //if local hour is greater than this
byte coffhour = 0;//but less than this, cathode poisoning routine will run

byte scycle = 0;  //variable that keeps track of things that happen every second vs 20x second.
byte ccycle = 0;  //digit for cathode poisoning routine
byte cdps = 0;    //cathode poising routine - turns decimal points on and off

byte channel = 1; //the current "channel" that is normally displayed on the nixie tubes (on startup)

byte neon = 0;    //bit0 control left neon, bit1 control right neon. Updated by nixie(). Overrides decimal.

byte pause = 0;   //if nonzero, pause cathode digit cycling.

byte daydec1 = 0;  //sets number of decimal places on right for days (for reference 1)
byte jifdec1 = 0;  //sets number of decimal places for jiffies 
byte daydec2 = 0;  //sets number of decimal places on right for days (for reference 2)
byte jifdec2 = 0;  //sets number of decimal places for jiffies

int64_t stopwatch_start = 0;
unsigned long stopwatch_t = 0;
boolean stopwatch_run = 0;

unsigned long lastmillis = 0;
char currentsecond = -1;

byte beeping = 0;
bool isBeeping=0;

DateTime utc;
DateTime local;
DateTime reference1;
DateTime reference2;

void setup (){
  pinMode(odatapin, OUTPUT);
  pinMode(oclockpin, OUTPUT);
  pinMode(olatchpin, OUTPUT);
  pinMode(ilatchpin, OUTPUT);
  pinMode(iclockpin, OUTPUT);
  pinMode(idatapin, INPUT);
  pinMode(buzzerpin, OUTPUT);
  pinMode(syncpin, INPUT_PULLUP);
  pinMode(WWVBpin, INPUT_PULLUP);
  
  Serial.begin(115200);
  myReceiver.enableIRIn(); // Start the receiver

  rtc.begin();
  rtc.writeSqwPinMode(0x00); //set odatapin3231 to output 1hz square wave
  //rtc.adjust(DateTime(F(__DATE__), F(__TIME__))+TimeSpan(0,0,0,8)); //set time
  utc = rtc.now();

  digitalWrite(iclockpin, LOW);
  digitalWrite(ilatchpin, HIGH);
  readconfig();

  Serial.println(F("Settings:"));
  Serial.print(F("localoffsethour   : ")); Serial.println(localoffsethour-127);
  Serial.print(F("localoffsetminute : ")); Serial.println(localoffsetminute-127);
  Serial.print(F("conhour           : ")); Serial.println(conhour);
  Serial.print(F("coffhour          : ")); Serial.println(coffhour);
  Serial.print(F("reference1        : ")); Serial.print(reference1.year());Serial.print(F("/"));Serial.print(reference1.month());Serial.print(F("/"));Serial.print(reference1.day());Serial.print(F(" "));Serial.print(reference1.hour());Serial.print(F(":"));Serial.print(reference1.minute());Serial.print(F(":"));Serial.println(reference1.second());
  
  stopwatch_start = utc.unixtime()*10 + cmillis()/100;
}

void loop(){
  utc = rtc.now();
  if(currentsecond != utc.second()){ lastmillis = millis(); currentsecond = utc.second(); }
  local = DateTime(utc + TimeSpan(0,localoffsethour-127,localoffsetminute-127,0));
  if(stopwatch_run){stopwatch_t = utc.unixtime()*10 + cmillis()/100 - stopwatch_start;}

  if(beeping>0){isBeeping=1;}
  switch(beeping){
    case 1:
      if(millis()%150<75){NewTone(buzzerpin, 1500);bitWrite(neon,7,1);}else{noNewTone(buzzerpin);bitWrite(neon,7,0);}
      break;
    case 2:
      NewTone(buzzerpin, 1500);
      break;
    default:
      if(isBeeping==1){noNewTone(buzzerpin);neon=0;}
      isBeeping=0;
      break;
  }
  
  if(local.hour()>=conhour && local.hour()<=coffhour && pause==0){                   //cycles digits at night or whenever
    cycledigits();
  }else{
    switch(channel){
      case 0: //UTC time
        nixie(utc.hour()*10000L+utc.minute()*100L+utc.second(), 0);
        break;
      case 1: //Local Time
        nixie(local.hour()*10000L+local.minute()*100L+local.second(), 0);
        break;
      case 2: //Winter Camp Time
        local = DateTime(utc + TimeSpan(0,-3,-43,0));  //can use local since it will be overwritten on next loop.
        nixie(local.hour()*10000L+local.minute()*100L+local.second(), 0);
        break;
      case 3: //Jiffies until next Winter Camp
        {DateTime tempdate = DateTime(utc.year(),12,27,15,43,0); //Start date 12,27,15,43,0 wintercamp start
        DateTime tempdate2 = DateTime(utc.year(),12,31,20,0,0);  //End date   12,31,20,0,0  wintercamp end
        if(utc.unixtime() >= tempdate2.unixtime()){tempdate = DateTime(utc.year()+1,12,27,15,43,0);} //alt Start date
        long diff = (utc.unixtime()*10 + cmillis()/100 - tempdate.unixtime()*10);
        float temp1 = floor(abs(diff / (9399.63716499)));
        byte tempdp = 4;
        if(temp1 >= 100){ tempdp = 3; }
        if(temp1 >= 1000){ tempdp = 2; }
        if(temp1 >= 10000){ tempdp = 1; }
        float temp = floor(diff / (9399.63716499*pow(10, -tempdp)));
        byte decimalmask;
        if(tempdp>0){bitSet(decimalmask, tempdp-1);}
        nixie(temp, decimalmask);}
        break;
      case 4: //TEST - Cycle digits  
        cycledigits();
        break;
      case 5: //Stopwatch display
        nixie(stopwatch_t, 1);
        break; 
      default: //default is local time
        channel = 1; 
        break;
    }  
  }
  
  byte inkey = irCheck();
  switch(inkey){
      case 10: //POWER
        channel = 4; //cycle digits test
        break;
      case 11: //SOURCE
        channel = 0; //UTC time
        break;
      case 12: //MUTE
        {if(pause==0){pause=30;}else{pause=0;}}
        break;
      case 13:
        {Serial.println(F("Record selection"));
        byte record = getIRbyte(0);
        Serial.print(F("Selected record ")); Serial.println(record);
        bitSet(neon, 7);
        switch(record){
          case 6:
            {byte temp = getIRbyte(utc.second()); utc = rtc.now();
            rtc.adjust(DateTime(utc.year(),utc.month(),utc.day(),utc.hour(),utc.minute(),temp));
            Serial.print(F("Clock second set: "));
            Serial.println(temp);}
            break;
          case 5:
           {byte temp = getIRbyte(utc.minute()); utc = rtc.now();
            rtc.adjust(DateTime(utc.year(),utc.month(),utc.day(),utc.hour(),temp,utc.second()));
            Serial.print(F("Clock minute set: "));
            Serial.println(temp);}
            break;  
          case 4:
            {byte temp = getIRbyte(utc.hour()); utc = rtc.now();
            rtc.adjust(DateTime(utc.year(),utc.month(),utc.day(),temp,utc.minute(),utc.second()));
            Serial.print(F("Clock hour set: "));
            Serial.println(temp);}
            break;  
          case 3:
            {byte temp = getIRbyte(utc.day()); utc = rtc.now();
            rtc.adjust(DateTime(utc.year(),utc.month(),temp,utc.hour(),utc.minute(),utc.second()));
            Serial.print(F("Clock day set: "));
            Serial.println(temp);}
            break;  
          case 2:
            {byte temp = getIRbyte(utc.month()); utc = rtc.now();
            rtc.adjust(DateTime(utc.year(),temp,utc.day(),utc.hour(),utc.minute(),utc.second()));
            Serial.print(F("Clock month set: "));
            Serial.println(temp);}
            break;  
          case 1:
            {int temp = getIRint(utc.year()); utc = rtc.now();
            rtc.adjust(DateTime(temp,utc.month(),utc.day(),utc.hour(),utc.minute(),utc.second()));
            Serial.print(F("Clock year set: "));
            Serial.println(temp);}
            break;
          case 7:  {int temp = getIRint(localoffsethour-127);   localoffsethour = 127+temp;   Serial.print(F("Local offset hour set: "));   Serial.println(temp);} break;
          case 8:  {int temp = getIRint(localoffsetminute-127); localoffsetminute = 127+temp; Serial.print(F("Local offset minute set: ")); Serial.println(temp);} break;
          case 9:  {conhour  = getIRbyte(conhour);  Serial.print(F("Cathode poisoning on set: "));  Serial.println(conhour); } break;
          case 10: {coffhour = getIRbyte(coffhour); Serial.print(F("Cathode poisoning off set: ")); Serial.println(coffhour);} break;
          case 11: secondsAdjust(); break;
          case 12:
            {int temp = getIRint(0);
            NewTone(buzzerpin, temp, 1000);}
            break;
         case 21: {int temp = getIRint(reference1.year());   reference1=DateTime(temp,reference1.month(),reference1.day(),reference1.hour(),reference1.minute(),reference1.second());  Serial.print(F("Reference1 year set: "));   Serial.println(temp);} break;
         case 22: {int temp = getIRint(reference1.month());  reference1=DateTime(reference1.year(),temp,reference1.day(),reference1.hour(),reference1.minute(),reference1.second());   Serial.print(F("Reference1 month set: "));  Serial.println(temp);} break;
         case 23: {int temp = getIRint(reference1.day());    reference1=DateTime(reference1.year(),reference1.month(),temp,reference1.hour(),reference1.minute(),reference1.second()); Serial.print(F("Reference1 day set: "));    Serial.println(temp);} break;
         case 24: {int temp = getIRint(reference1.hour());   reference1=DateTime(reference1.year(),reference1.month(),reference1.day(),temp,reference1.minute(),reference1.second());  Serial.print(F("Reference1 hour set: "));   Serial.println(temp);} break;
         case 25: {int temp = getIRint(reference1.minute()); reference1=DateTime(reference1.year(),reference1.month(),reference1.day(),reference1.hour(),temp,reference1.second());    Serial.print(F("Reference1 minute set: ")); Serial.println(temp);} break;
         case 26: {int temp = getIRint(reference1.second()); reference1=DateTime(reference1.year(),reference1.month(),reference1.day(),reference1.hour(),reference1.minute(),temp);    Serial.print(F("Reference1 second set: ")); Serial.println(temp);} break;
         case 27: {daydec1 = getIRbyte(daydec1); } break;
         case 28: {jifdec1 = getIRbyte(jifdec1); } break;
         case 31: {int temp = getIRint(reference2.year());   reference2=DateTime(temp,reference2.month(),reference2.day(),reference2.hour(),reference2.minute(),reference2.second());  Serial.print(F("reference2 year set: "));   Serial.println(temp);} break;
         case 32: {int temp = getIRint(reference2.month());  reference2=DateTime(reference2.year(),temp,reference2.day(),reference2.hour(),reference2.minute(),reference2.second());   Serial.print(F("reference2 month set: "));  Serial.println(temp);} break;
         case 33: {int temp = getIRint(reference2.day());    reference2=DateTime(reference2.year(),reference2.month(),temp,reference2.hour(),reference2.minute(),reference2.second()); Serial.print(F("reference2 day set: "));    Serial.println(temp);} break;
         case 34: {int temp = getIRint(reference2.hour());   reference2=DateTime(reference2.year(),reference2.month(),reference2.day(),temp,reference2.minute(),reference2.second());  Serial.print(F("reference2 hour set: "));   Serial.println(temp);} break;
         case 35: {int temp = getIRint(reference2.minute()); reference2=DateTime(reference2.year(),reference2.month(),reference2.day(),reference2.hour(),temp,reference2.second());    Serial.print(F("reference2 minute set: ")); Serial.println(temp);} break;
         case 36: {int temp = getIRint(reference2.second()); reference2=DateTime(reference2.year(),reference2.month(),reference2.day(),reference2.hour(),reference2.minute(),temp);    Serial.print(F("reference2 second set: ")); Serial.println(temp);} break;
         case 37: {daydec2 = getIRbyte(daydec2); } break;
         case 38: {jifdec2 = getIRbyte(jifdec2); } break;
         case 60:
           nixie(0, 192);
           WWVBsync();
           break;
         case 254:
           readconfig();
           break;
         case 255:
           writeconfig();
           break;
         }
        bitClear(neon, 7);
        NewTone(buzzerpin, 500, 100);
        }break;
      case 14: //CH+
        channel = 2; // Winter Camp Time
        break;
      case 15: //TIME SHIFT
        break;
      case 16: //VOL- //reset and start stopwatch
        stopwatch_start = utc.unixtime()*10 + cmillis()/100;
        stopwatch_run = 1;
        channel = 5;
        break;
      case 17: //FULLSCREEN
        channel = 1; //local time
        break;
      case 18: //VOL+ //start/stop stopwatch
        stopwatch_run = !stopwatch_run;
        channel = 5;
        break;
      case 0:  //0 KEY
        digitalWrite(buzzerpin, 1); delay(500); digitalWrite(buzzerpin, 0);
        break;
      case 19: //CH-
        channel = 3; //Jiffy clock
        break;
      case 20: //RECALL
        break;
      case 1:  //1 KEY
        break;
      case 2:  //2 KEY
        break;
      case 3:  //3 KEY
        break;
      case 4:  //4 KEY
        break;
      case 5:  //5 KEY
        break;
      case 6:  //6 KEY
        break;
      case 7:  //7 KEY
        break;
      case 8:  //8 KEY
        break;
      case 9:  //9 KEY
        break;
    }

  //if(inkey == 20){for(byte i = 0; i<=8; i++){NewTone(buzzerpin, 2000, 50);delay(50);NewTone(buzzerpin, 2000, 50);delay(50);NewTone(buzzerpin, 2000, 50);delay(50);NewTone(buzzerpin, 2000, 50);delay(600);}}
  
  scycle++;
  if(scycle==20){ scycle = 0; //runs every second
    if(pause>0){pause--;}
    //Serial.print(F("Local: "));Serial.print(local.year(), DEC);Serial.print('/');Serial.print(local.month(), DEC);Serial.print('/');Serial.print(local.day(), DEC);Serial.print(" ");Serial.print(local.hour(), DEC);Serial.print(':');Serial.print(local.minute(), DEC);Serial.print(':');Serial.println(local.second(), DEC);  
    //Serial.print(F("UTC:   "));Serial.print(utc.year(), DEC);Serial.print('/');Serial.print(utc.month(), DEC);Serial.print('/');Serial.print(utc.day(), DEC);Serial.print(" ");Serial.print(utc.hour(), DEC);Serial.print(':');Serial.print(utc.minute(), DEC);Serial.print(':');Serial.print(utc.second(), DEC);Serial.print(' ');Serial.println(utc.unixtime());
    //Serial.print(F("Ref:   "));Serial.print(reference.year(), DEC);Serial.print('/');Serial.print(reference.month(), DEC);Serial.print('/');Serial.print(reference.day(), DEC);Serial.print(" ");Serial.print(reference.hour(), DEC);Serial.print(':');Serial.print(reference.minute(), DEC);Serial.print(':');Serial.print(reference.second(), DEC);Serial.print(' ');Serial.println(reference.unixtime());
    //Serial.println(diff, DEC);
  }
  
  delay(20);
}



void nixie(long number, byte decimal){
    if(bitRead(neon, 6)){bitSet(decimal, 6);} //left neon
    if(bitRead(neon, 7)){bitSet(decimal, 7);} //right neon
    if(number < 0){bitSet(decimal, 6);}
    number = abs(number);
    byte hthou = number / 100000L;
    number = number-hthou*100000L;
    byte tthou = number / 10000L;
    number = number-tthou*10000L;
    byte thou = number / 1000L;
    number = number-thou*1000L;
    byte hund = number / 100L;
    number = number-hund*100L;
    byte tens = number / 10L;
    number = number-tens*10L;
    byte ones = number;

    ones = ones + 16*tens;  //combined ones and tens
    hund = hund + 16*thou;
    tthou = tthou + 16*hthou;

    digitalWrite(oclockpin, LOW); //latch low
    shiftOut(odatapin, olatchpin, MSBFIRST, decimal);
    shiftOut(odatapin, olatchpin, MSBFIRST, tthou);
    shiftOut(odatapin, olatchpin, MSBFIRST, hund);
    shiftOut(odatapin, olatchpin, MSBFIRST, ones);
    digitalWrite(oclockpin, HIGH); //latch high
}

int getIRint(int existing){
  int current = 0;
  while(1){
    nixie(current, 0);
    Serial.println(current, DEC);
    byte inkey = 255; while(inkey == 255){ inkey = irCheck(); }
    if(inkey == 17){return current;}               //assembling the number is done, return it.
    if(inkey == 20){current = existing;}           //user pressed RECALL. let current equal the existing number
    if(inkey == 15){current = -current;}           //user pressed TIMESHIFT (using as +/- key);
    if(inkey == 12){current = 0;}                  //user pressed MUTE (clear). Reset to zero.
    if(inkey == 14){current++;}                    //channel up/down buttons add/subtract      
    if(inkey == 19){current--;}
    if(inkey < 10){if(current>=0){current = 10*current + inkey;}else{current = 10*current - inkey;}}  //a number was entered. Append it, and manage minus sign;
  }
}

byte getIRbyte(byte existing){
  byte current = 0;
  while(1){
    nixie(current, 0);
    Serial.println(current, DEC);
    byte inkey = 255; while(inkey == 255){ inkey = irCheck(); }
    if(inkey == 17){return current;}               //assembling the number is done, return it.
    if(inkey == 20){current = existing;}           //user pressed RECALL. let current equal the existing number
    if(inkey == 12){current = 0;}                  //user pressed MUTE (clear). Reset to zero.
    if(inkey == 14){current++;}                    //channel up/down buttons add/subtract      
    if(inkey == 19){current--;}
    if(inkey < 10){current = 10*current + inkey;}  //a number was entered. Append it.
    Serial.println(inkey, DEC);
  }
}

void secondsAdjust(){
  byte current = 0;
  byte neonMask = 0; 
  byte syncen = 0;
  byte inkey = 255;
  while(1){
    if(inkey == 255){ inkey = irCheck(); }
    utc = rtc.now();
    if(currentsecond != utc.second()){
      lastmillis = millis(); currentsecond = utc.second(); syncen--;
      if(utc.second()==0){NewTone(buzzerpin, 1000, 800);}else{NewTone(buzzerpin, 1000, 5);}
      if(syncen==255){syncen=0;}
      if(inkey == 14){rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())+TimeSpan(0,0,0,1)); inkey = 255;}                    //channel up/down buttons add/subtract seconds      
      if(inkey == 19){rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())-TimeSpan(0,0,0,1)); inkey = 255;}
      }
    if((digitalRead(syncpin)==0&&syncen==0)||(inkey == 11)){
      inkey=255;
      syncen=1;
      int fromlast = cmillis();
      if(fromlast<999-fromlast){Serial.println(fromlast);rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second()));}else{Serial.println(-999+fromlast);rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())+TimeSpan(0,0,0,1));}
      }
    if(cmillis()<50){neonMask = 64;}else{neonMask = 0;}
    nixie(utc.hour()*10000L+utc.minute()*100L+utc.second(), neonMask); 
    if(inkey == 17){digitalWrite(buzzerpin, 0); return;}               //User pressed enter. Done
  }
}



byte irCheck(){                                //Check for a valid code
  if (myReceiver.getResults()) { 
    myDecoder.decode();                        //Decode ir code
    unsigned long rxcode=myDecoder.value;
    byte decoded=255;
    switch(rxcode){
      case 0xff48b7:
       decoded=0;
       break;
      case 0xff906f:
       decoded=1;
       break;
      case 0xffb847:
       decoded=2;
       break;
      case 0xfff807:
       decoded=3;
       break;
      case 0xffb04f:
       decoded=4;
       break;
      case 0xff9867:
       decoded=5;
       break;
      case 0xffd827:
       decoded=6;
       break;
      case 0xff8877:
       decoded=7;
       break;
      case 0xffa857:
       decoded=8;
       break;
      case 0xffe817:
       decoded=9;
       break;
      case 0xffb24d:
       decoded=10; //power button
       break;
      case 0xff2ad5:
       decoded=11; //source button
       break;
      case 0xff6897:
       decoded=12; //mute
       break;
      case 0xff32cd:
       decoded=13; //record
       break;
      case 0xffa05f:
       decoded=14; //ch+
       break;
      case 0xff30cf:
       decoded=15; //time shift
       break;
      case 0xff50af:
       decoded=16; //vol-
       break;
      case 0xff02fd:
       decoded=17; //full screen
       break;
      case 0xff7887:
       decoded=18; //vol+
       break;
      case 0xff40bf:
       decoded=19; //ch-
       break;
      case 0xff38c7:
       decoded=20; //recall
       break;
    }
    delay(20);
    myReceiver.enableIRIn();                   //Restart receiver
    delay(20);
    return decoded;
  } else { return 255; }
}

byte read165(){
  byte rin = 0;
  digitalWrite(ilatchpin, LOW);
  delayMicroseconds(5);
  digitalWrite(ilatchpin, HIGH);
  for(byte i = 0; i < 8; i++){
    bitWrite(rin, i, digitalRead(idatapin));
    digitalWrite(iclockpin, HIGH);
    delayMicroseconds(5);
    digitalWrite(iclockpin, LOW);
  }
   return(rin);
}

void readconfig(){
  localoffsethour   = EEPROM.read(10);
  localoffsetminute = EEPROM.read(11);
  conhour  = EEPROM.read(12);
  coffhour = EEPROM.read(13);
  reference1 = DateTime(EEPROM.read(14)+2000,EEPROM.read(15),EEPROM.read(16),EEPROM.read(17),EEPROM.read(18),EEPROM.read(19));
  daydec1 = EEPROM.read(20);
  jifdec1 = EEPROM.read(21);
  reference2 = DateTime(EEPROM.read(22)+2000,EEPROM.read(23),EEPROM.read(24),EEPROM.read(25),EEPROM.read(26),EEPROM.read(27));
  daydec2 = EEPROM.read(28);
  jifdec2 = EEPROM.read(29);
  NewTone(buzzerpin, 1000, 100); delay(100);
  NewTone(buzzerpin, 1250, 100); delay(100);
  NewTone(buzzerpin, 1500, 100); delay(100);
}

void writeconfig(){
  EEPROM.update(10, localoffsethour);
  EEPROM.update(11, localoffsetminute);
  EEPROM.update(12, conhour);
  EEPROM.update(13, coffhour);
  EEPROM.update(14, reference1.year()-2000);EEPROM.update(15, reference1.month());EEPROM.update(16, reference1.day());EEPROM.update(17, reference1.hour());EEPROM.update(18, reference1.minute());EEPROM.update(19, reference1.second());
  EEPROM.update(20, daydec1);
  EEPROM.update(21, jifdec1);
  EEPROM.update(22, reference2.year()-2000);EEPROM.update(23, reference2.month());EEPROM.update(24, reference2.day());EEPROM.update(25, reference2.hour());EEPROM.update(26, reference2.minute());EEPROM.update(27, reference2.second());
  EEPROM.update(28, daydec2);
  EEPROM.update(29, jifdec2);
  NewTone(buzzerpin, 1500, 100); delay(100);
  NewTone(buzzerpin, 1250, 100); delay(100);
  NewTone(buzzerpin, 1000, 100); delay(100);
}

void cycledigits(){
  ccycle++;
     if(ccycle>=10){
       ccycle=0;
       if(cdps == 0){cdps = 0b11111111;}else{cdps = 0;}
     }
     nixie(ccycle*111111L, cdps);
     delay(500);
}

int cmillis(){
  long temp = millis() -  lastmillis;
  return temp;
}
