/*For a Nixie clock prototype - Alnwlsn 2017
 *Clock uses three 74hc595 shift registers in series to control six nixie tubes via for k155id1 driver ics.  
 *Most signifigant information sent out first.
 *ds3231 for timekeeping.
 *
 *Clock normally displays the time but from 2 - 5 am it cycles all nixie digits to reduce cathode poisioning.
 *
 *Connections (arduino - item)
 * A4 - SDA on odatapin3231
 * A5 - SCL on odatapin3231
 */
 
#include <Wire.h>
#include "RTClib.h"
#include <EEPROM.h>

#define odatapin     10
#define oclockpin    11
#define olatchpin    12
#define ilatchpin     5
#define iclockpin     6
#define idatapin      7
#define buzzerpin     9

RTC_DS3231 rtc;

unsigned long unixref = 0;
long diff = 0;
byte istate = 0;
byte istateold = 0;
byte offsethour = 127;
byte offsetminute = 127;

byte ryear = 0;
byte rmonth = 0;
byte rday = 0;
byte rhour = 0;
byte rminute = 0;
byte rsecond = 0;

byte conhour = 13;
byte coffhour = 13;

byte scycle = 0; //variable that keeps track of things that happen every second vs 20x second.
byte ccycle = 0; //digit for cathode poisoning routine
byte cdps = 0;  //cathode poising routine - turns decimal points on and off

DateTime utc;
DateTime local;
DateTime reference;

void setup () {
  pinMode(odatapin, OUTPUT);
  pinMode(oclockpin, OUTPUT);
  pinMode(olatchpin, OUTPUT);
  pinMode(ilatchpin, OUTPUT);
  pinMode(iclockpin, OUTPUT);
  pinMode(idatapin, INPUT);
  pinMode(buzzerpin, OUTPUT);
  
  Serial.begin(9600);

  rtc.begin();
  rtc.writeSqwPinMode(0x00); //set odatapin3231 to output 1hz square wave
  //rtc.adjust(DateTime(F(__DATE__), F(__TIME__))+TimeSpan(0,0,0,8)); //set time
  utc = rtc.now();

  digitalWrite(iclockpin, LOW);
  digitalWrite(ilatchpin, HIGH);
  readconfig();
}

void loop(){
  istate = read165();
  utc = rtc.now();
  local = DateTime(utc + TimeSpan(0,offsethour-127,offsetminute-127,0));
  reference = DateTime(ryear+2000, rmonth, rday, rhour, rminute, rsecond);
  diff = utc.unixtime() - reference.unixtime();

  if(istate<60 && local.hour()>=conhour && local.hour()<=coffhour){istate=60;} //cycles digits at night or whenever
  
  if(istate != istateold){ //runs only if state is different (set)
    istateold = istate;
    switch(istate){
      case 69:
       rtc.adjust(DateTime(utc.year()-1,utc.month(),utc.day(),utc.hour(),utc.minute(),utc.second()));utc=rtc.now();
       nixie(utc.year(), 0);
       break;
      case 70:
       rtc.adjust(DateTime(utc.year()+1,utc.month(),utc.day(),utc.hour(),utc.minute(),utc.second()));utc=rtc.now();
       nixie(utc.year(), 0);
       break;
      case 73:
       {byte tmonth = utc.month(); tmonth--; int tyear = utc.year(); if(tmonth <= 0){tmonth = 12; tyear--;}
       rtc.adjust(DateTime(tyear,tmonth,utc.day(),utc.hour(),utc.minute(),utc.second()));utc=rtc.now(); }
       nixie(utc.month(), 0);
       break;
      case 74:
       {byte tmonth = utc.month(); tmonth++; int tyear = utc.year(); if(tmonth >= 13){tmonth = 1; tyear++;}
       rtc.adjust(DateTime(tyear,tmonth,utc.day(),utc.hour(),utc.minute(),utc.second()));utc=rtc.now(); }
       nixie(utc.month(), 0);
       break;
      case 77:
       rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())-TimeSpan(1,0,0,0));
       nixie(utc.day(), 0);
       break;
      case 78:
       rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())+TimeSpan(1,0,0,0));
       nixie(utc.day(), 0);
       break;
      case 81:
       rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())-TimeSpan(0,1,0,0));
       nixie(utc.hour(), 0);
       break;
      case 82:
       rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())+TimeSpan(0,1,0,0));
       nixie(utc.hour(), 0);
       break;
      case 85:
       rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())-TimeSpan(0,0,1,0));
       nixie(utc.minute(), 0);
       break;
      case 86:
       rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())+TimeSpan(0,0,1,0));
       nixie(utc.minute(), 0);
       break;
      case 89:
       rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())-TimeSpan(0,0,0,1));
       nixie(utc.second(), 0);
       break;
      case 90:
       rtc.adjust(DateTime(utc.year(), utc.month(), utc.day(), utc.hour(), utc.minute(), utc.second())+TimeSpan(0,0,0,1));
       nixie(utc.second(), 0);
       break;
      case 93:
       offsethour--;
       nixie(offsethour-127, 0);
       break;
      case 94:
       offsethour++;
       nixie(offsethour-127, 0);
       break;
      case 97:
       offsetminute--;
       nixie(offsetminute-127, 0);
       break;
      case 98:
       offsetminute++;
       nixie(offsetminute-127, 0);
       break;
      case 101:
       conhour--;
       nixie(conhour, 0);
       break;
      case 102:
       conhour++;
       nixie(conhour, 0);
       break;
      case 105:
       coffhour--;
       nixie(coffhour, 0);
       break;
      case 106:
       coffhour++;
       nixie(coffhour, 0);
       break;
      case 253:
       readconfig();
       break;
      case 254:
       writeconfig();
       break;
      default:
       nixie(istate+404000L, 0);
       break;
    }
  }
  
  switch(istate){ //runs every time (live)
    case 0: //local time
     nixie(local.hour()*10000L+local.minute()*100L+local.second(), 0);
     break;
    case 4: //utc time
     nixie(utc.hour()*10000L+utc.minute()*100L+utc.second(), 0);
     break;
    case 8:
     {DateTime wcst = DateTime(utc - TimeSpan(0,3,43,0));
     nixie(wcst.hour()*10000L+wcst.minute()*100L+wcst.second(), 0);}
     break;
    case 12: //days until ref
     {float temp = diff / 86.4 ; nixie(temp, 0b00000100);}
     break;
    case 16: //tenths of jiffies
     {float temp = diff/93.9963716499; nixie(temp, 0b00000001);}
     break;
    case 20: //hundredths of jiffies
     {float temp = diff/.939963716499; nixie(temp, 0b00000100);}
     break;
    case 60:
     ccycle++;
     if(ccycle>=10){
       ccycle=0;
       if(cdps == 0){cdps = 0b11111111;}else{cdps = 0;}
     }
     nixie(ccycle*111111L, cdps);
     delay(500);
     break;
    case 68:
     nixie(utc.year(), 0);
     break;
    case 72:
     nixie(utc.month(), 0);
     break;
    case 76:
     nixie(utc.day(), 0);
     break;
    case 80:
     nixie(utc.hour(), 0);
     break;
    case 84:
     nixie(utc.minute(), 0);
     break;
    case 88:
     nixie(utc.second(), 0);
     break;
    case 92:
     nixie(offsethour-127, 0);
     break;
    case 96:
     nixie(offsetminute-127, 0);
     break;
    case 100:
     nixie(conhour, 0);
     break;
    case 104:
     nixie(coffhour, 0);
     break;
       
  }

  scycle++;
  if(scycle==20){ scycle = 0; //runs every second
    Serial.print(F("Local: "));Serial.print(local.year(), DEC);Serial.print('/');Serial.print(local.month(), DEC);Serial.print('/');Serial.print(local.day(), DEC);Serial.print(" ");Serial.print(local.hour(), DEC);Serial.print(':');Serial.print(local.minute(), DEC);Serial.print(':');Serial.println(local.second(), DEC);  
    Serial.print(F("UTC:   "));Serial.print(utc.year(), DEC);Serial.print('/');Serial.print(utc.month(), DEC);Serial.print('/');Serial.print(utc.day(), DEC);Serial.print(" ");Serial.print(utc.hour(), DEC);Serial.print(':');Serial.print(utc.minute(), DEC);Serial.print(':');Serial.print(utc.second(), DEC);Serial.print(' ');Serial.println(utc.unixtime());
    Serial.print(F("Ref:   "));Serial.print(reference.year(), DEC);Serial.print('/');Serial.print(reference.month(), DEC);Serial.print('/');Serial.print(reference.day(), DEC);Serial.print(" ");Serial.print(reference.hour(), DEC);Serial.print(':');Serial.print(reference.minute(), DEC);Serial.print(':');Serial.print(reference.second(), DEC);Serial.print(' ');Serial.println(reference.unixtime());
    Serial.println(diff, DEC);
  }
  delay(50);
}



void nixie(long number, byte decimal){
    if(number < 0){bitSet(decimal, 6);}
    number = abs(number);
    byte hthou = number / 100000L;
    number = number-hthou*100000L;
    byte tthou = number / 10000L;
    number = number-tthou*10000L;
    byte thou = number / 1000L;
    number = number-thou*1000L;
    byte hund = number / 100L;
    number = number-hund*100L;
    byte tens = number / 10L;
    number = number-tens*10L;
    byte ones = number;

    ones = ones + 16*tens;  //combined ones and tens
    hund = hund + 16*thou;
    tthou = tthou + 16*hthou;

    digitalWrite(oclockpin, LOW); //latch low
    shiftOut(odatapin, olatchpin, MSBFIRST, decimal);
    shiftOut(odatapin, olatchpin, MSBFIRST, tthou);
    shiftOut(odatapin, olatchpin, MSBFIRST, hund);
    shiftOut(odatapin, olatchpin, MSBFIRST, ones);
    digitalWrite(oclockpin, HIGH); //latch high
}

byte read165(){
  byte rin = 0;
  digitalWrite(ilatchpin, LOW);
  delayMicroseconds(5);
  digitalWrite(ilatchpin, HIGH);
  for(byte i = 0; i < 8; i++){
    bitWrite(rin, i, digitalRead(idatapin));
    digitalWrite(iclockpin, HIGH);
    delayMicroseconds(5);
    digitalWrite(iclockpin, LOW);
  }
   return(rin);
}

void readconfig(){
  offsethour   = EEPROM.read(256);
  offsetminute = EEPROM.read(257);
  ryear    = EEPROM.read(258);
  rmonth   = EEPROM.read(259);
  rday     = EEPROM.read(260);
  rhour    = EEPROM.read(261);
  rminute  = EEPROM.read(262);
  rsecond  = EEPROM.read(263);
  conhour  = EEPROM.read(264);
  coffhour = EEPROM.read(265);
  tone(buzzerpin, 1000);
  delay(100);
  tone(buzzerpin, 2000);
  delay(100);
  noTone(buzzerpin);
}

void writeconfig(){
  EEPROM.update(256, offsethour);
  EEPROM.update(257, offsetminute);
  EEPROM.update(258, ryear);
  EEPROM.update(259, rmonth);
  EEPROM.update(260, rday);
  EEPROM.update(261, rhour);
  EEPROM.update(262, rminute);
  EEPROM.update(263, rsecond);
  EEPROM.update(264, conhour);
  EEPROM.update(265, coffhour);
  tone(buzzerpin, 2000);
  delay(100);
  tone(buzzerpin, 1000);
  delay(100);
  noTone(buzzerpin);
}
